
//Surfaces formulas from...
//http://local.wasp.uwa.edu.au/~pbourke/surfaces_curves/
//http://mathworld.wolfram.com/topics/Surfaces.html

//shading:         phong
//lighting model:  blinn
//lighting type:   directional

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tV  : VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tWV : WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;

float4x4 TT ;

//light properties
float3 lDir <string uiname="Light Direction";> = {0, -5, 2};        //light direction in world space
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 25.0;     //shininess of specular highlight

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 PosWVP    : POSITION;
    float4 TexCd     : TEXCOORD0;
    float3 LightDirV : TEXCOORD1;
    float2 UV        : TEXCOORD2;
    float3 ViewDirV  : TEXCOORD3;
    float3 Pos       : TEXCOORD4;
};

#define TWOPI 6.28318531
#define PI    3.14159265

// --------------------------------------------------------------------------------------------------
// FUNCTIONS:
// --------------------------------------------------------------------------------------------------

float gridSpaceX;
float gridSpaceY;

float gridOffsetX;
float gridOffsetY;

float gridScaleX = 1;
float gridScaleY = 1;

float param1 = 1;
float param2 = 1;
float param3 = 1;
float param4 = 1;

////////////////////////////////////////////////////////////////////////////////
/*Steinbach Screw
x= u cos v
y= u sin v
z= v cos u
*///////////////////////////////////////////////////////////////////////////////

float3 SteinbachScrew(float u, float v)
{
       float x = param1 * (u) * cos(v);
       float y = param1 * (u) * sin(v);
       float z = param1 * (v) * cos(u);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Sine surface
x= sin(u)
y= sin(v)
z= sin(u+v)
*///////////////////////////////////////////////////////////////////////////////

float3 Sine(float u, float v)
{
       float x = param1 * sin(u);
       float y = param1 * sin(v);
       float z = param1 * sin(u+v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Paraboloid
x= a (u/H)^0,5 cos(v)
y= a (u/H)^0,5 sin(v)
z= u
0<u<infinty,0<v<2pi
*///////////////////////////////////////////////////////////////////////////////

float3 Paraboloid(float u, float v)
{
       float x = param1 * pow((u/param2),0.5) * cos(v);
       float y = param1 * pow((u/param2),0.5) * sin(v);
       float z = (u);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Hyperbolic Helicoid
x= (sin(v) cos(T u)) / (1+cosh(u) cosh(v))
y= (sin(v) sin(T u)) / (1+cosh(u) cosh(v))
z= (cosh(v) sinh(u)) / (1+cosh(u) cosh(v))
*///////////////////////////////////////////////////////////////////////////////

float3 HyperHelicoid(float u, float v)
{
       float x = param1 * (sin(v) * cos(param3 * u)) / (1 + cosh(u) * cosh(v));
       float y = param1 * (sin(v) * sin(param3 * u)) / (1 + cosh(u) * cosh(v));
       float z = (cosh(v) * sinh(u)) / (1 + cosh(u) * cosh(v));

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Hyperboloid
x= a (1+u2)^0,5 cos(v)
y= b (1+u2)^0,5 sin(v)
z= c (u)
0<u< 2pi, 0<v<2pi
*///////////////////////////////////////////////////////////////////////////////

float3 Hyperboloid(float u, float v)
{
       float x = param1 * pow((1 + pow(u,2)),0.5) * cos(v);
       float y = param2 * pow((1 + pow(u,2)),0.5) * sin(v);
       float z = param3 * (u);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Eight Surface
x= cos(u) sin(2v)
y= sin (u) sin(2v)
z= sin (v)
0<u< 2pi, -pi/2<v<pi/2
*///////////////////////////////////////////////////////////////////////////////

float3 Eight(float u, float v)
{
       float x = cos(u) * sin(2*v);
       float y = sin(u) * sin(2*v);
       float z = sin(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Corkscrew Surface or Twisted Sphere
x= cos(u) cos(v)
y= sin(u) cos(v)
z= sin(v) + b u
0<s<pi, 0<t<2pi
*///////////////////////////////////////////////////////////////////////////////

float3 Corkscrew(float u, float v)
{
       float x = param1 * cos(u) * cos(v);
       float y = param1 * sin(u) * cos(v);
       float z = param1 * sin(v) + param2 * u;

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Roman and boy surfaces
x=(sqrt(2)cos(2u)cos^2v+cosusin(2v))/(2-alphasqrt(2)sin(3u)sin(2v))
y=(sqrt(2)sin(2u)cos^2v-sinusin(2v))/(2-alphasqrt(2)sin(3u)sin(2v))
z=(3cos^2v)/(2-alphasqrt(2)sin(3u)sin(2v))
*///////////////////////////////////////////////////////////////////////////////

float3 Roman2Boy(float u, float v)
{
       float x = (pow(2,0.5) * cos(2*u) * pow(cos(v),2) + cos(u) * sin(2*v)) / (2 - param1*pow(2,0.5) * sin(3*u) * sin(2*v));
       float y = (pow(2,0.5) * sin(2*u) * pow(cos(v),2) + sin(u) * sin(2*v)) / (2 - param1*pow(2,0.5) * sin(3*u) * sin(2*v));
       float z = (3*pow(cos(v),2)) / (2 - param1*pow(2,0.5) * sin(3*u) * sin(2*v));
        //when param1 = 0 = Roman surface/when = 1 = Boy surface
      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Twisted Pipe Surface/////
x = cos(v) * (2 + cos(u)) / sqrt(1 + sin(v)2)
y = sin(v + 2pi/3) * (2 + cos(u + 2pi/3)) / sqrt(1 + sin(v)2)
z = sin(v - 2pi/3) * (2 + cos(u - 2pi/3)) / sqrt(1 + sin(v)2)
0 <= u <= 2 pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 TwistedPipe(float u, float v)
{
       float x = cos(v) * (2 + cos(u)) / sqrt(1 + pow(sin(v),2));
       float y = sin(v + TWOPI/3) * (2 + cos(u + TWOPI/3)) / sqrt(1 + pow(sin(v),2));
       float z = sin(v - TWOPI/3) * (2 + cos(u - TWOPI/3)) / sqrt(1 + pow(sin(v),2));

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Stiletto surface/////
x = x = (2 + cos(u)) cos(v)3 sin(v)
y = (2 + cos(u+2pi/3)) cos(v+2pi/3)2 sin(v+2pi/3)2
z = -(2 + cos(u-2pi/3)) cos(v+2pi/3)2 sin(v+2pi/3)2
0 <= u <= 2 pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 Stiletto(float u, float v)
{
       float x = (2 + cos(u)) * pow(cos(v),3) * sin(v);
       float y = (2 + cos(u + TWOPI/3)) * pow(cos(v + TWOPI/3),2) * pow(sin(v + TWOPI/3),2);
       float z = -(2 + cos(u - TWOPI/3)) * pow(cos(v + TWOPI/3),2) * pow(sin(v + TWOPI/3),2);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Bohemian Dome/////
x = a cos(u)
y = a sin(u) + b cos(v)
z = c sin(v)
0 <= u <= 2 pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 BohemianDome(float u, float v)
{
       float x = param1 * cos(u);
       float y = param1 * sin(u) + param2 * cos(v);
       float z = param3 * sin(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Folium/////
x = cos(u) (2 v/ pi - tanh(v))
y = cos(u + 2 pi / 3) / cosh(v)
z = cos(u - 2 pi / 3) / cosh(v)
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Folium(float u, float v)
{
       float x = cos(u) * (2 * v/ PI - tanh(v));
       float y = cos(u + TWOPI / 3) / cosh(v);
       float z = cos(u - TWOPI / 3) / cosh(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Maeder's Owl/////
x = v cos(u) - 0.5 v2 cos(2 u)
y = - v sin(u) - 0.5 v2 sin(2 u)
z = 4 v1.5 cos(3 u / 2) / 3
0 <= u <= 4pi  and 0 <= v <= 1
*///////////////////////////////////////////////////////////////////////////////

float3 MaedersOwl(float u, float v)
{
       float x = v * cos(u) - 0.5 * pow(v,2) * cos(2 * u);
       float y = - v * sin(u) - 0.5 * pow(v,2) * sin(2 * u);
       float z = 4 * pow(v,1.5) * cos(3 * u / 2) / 3;

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Astroidal Ellipsoid/////
x = (cos(u) cos(v))^3
y = (sin(u) cos(v))^3
z = sin(v)^3
0 <= u <= pi  and 0 <= v <= 2pi
*///////////////////////////////////////////////////////////////////////////////

float3 AstroidalEllipsoid(float u, float v)
{
       float x = pow(cos(u)*cos(v),3);
       float y = pow(sin(u)*cos(v),3);
       float z = pow(sin(v),3);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Triaxial Tritorus/////
x = sin(u) (1 + cos(v))
y = sin(u + 2pi / 3) (1 + cos(v + 2pi / 3))
z = sin(u + 4pi / 3) (1 + cos(v + 4pi / 3))
-pi <= u <=pi and -pi <= v <=pi
*///////////////////////////////////////////////////////////////////////////////

float3 TriaxialTritorus(float u, float v)
{
       float x = sin(u) * (1 + cos(v));
       float y = sin(u + TWOPI / 3) * (1 + cos(v + TWOPI / 3));
       float z = sin(u + 2*TWOPI / 3) * (1 + cos(v + 2*TWOPI / 3));

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Limpet Torus/////
x = cos(u) / [sqrt(2) + sin(v)]
y = sin(u) / [sqrt(2) + sin(v)]
z = 1 / [sqrt(2) + cos(v)]
*///////////////////////////////////////////////////////////////////////////////

float3 LimpetTorus(float u, float v)
{
       float x = cos(u) / (sqrt(2) + sin(v));
       float y = sin(u) / (sqrt(2) + sin(v));
       float z = 1 / (sqrt(2) + cos(v));

      return float3(x, y, z);
}
////////////////////////////////////////////////////////////////////////////////
/*Triaxial Hexatorus/////
x = sin(u) / [sqrt(2) + cos(v)]
y = sin(u+2pi/3) / [sqrt(2) + cos(v+2pi/3)]
z = cos(u-2pi/3) / [sqrt(2) + cos(v-2pi/3)]
*///////////////////////////////////////////////////////////////////////////////

float3 TriaxialHexatorus(float u, float v)
{
       float x = sin(u) / (sqrt(2) + cos(v));
       float y = sin(u + TWOPI/3) / (sqrt(2) + cos(v + TWOPI/3));
       float z = cos(u - TWOPI/3) / (sqrt(2) + cos(v - TWOPI/3));

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Cross Cap/////
x = cos(u) sin(2v)
y = sin(u) sin(2v)
z = cos(v)^2 - cos(u)^2  sin(v)^2
0 <= u <= 2pi and 0 <= v <= pi/ 2
*///////////////////////////////////////////////////////////////////////////////

float3 CrossCap(float u, float v)
{
       float x = cos(u) * sin(2*v);
       float y = sin(u) * sin(2*v);
       float z = pow(cos(v), 2) - pow(cos(u),2) * pow (sin(v),2);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Handkerchief Surface/////
x = u
y = v
z = u3 / 3 + u v2 + a (u2 - v2)
-1 <= u <= 1, -1 <= v <= 1
*///////////////////////////////////////////////////////////////////////////////

float3 Handkerchief(float u, float v)
{

       float x = u;
       float y = v;
       float z = pow(u,3) / 3 + u * pow(v,2) + param1 * (pow(u,2) - pow(v,2));

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Bow Curve/////
x = (2 + T sin(2 pi u)) sin(4 pi v)
y = (2 + T sin(2 pi u)) cos(4 pi v)
z = T cos(2 pi u) + 3 cos(2 pi v)
0 <= u <= 1, 0 <= v <= 1, 0 < T (Thickness)
*///////////////////////////////////////////////////////////////////////////////

float3 Bow(float u, float v)
{

       float x = (2 + param1 * sin(TWOPI * u)) * sin(2 * TWOPI * v);
       float y = (2 + param1 * sin(TWOPI * u)) * cos(2 * TWOPI * v);
       float z = param1 * cos(TWOPI * u) + 3 * cos(TWOPI * v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Apple Surface/////
x = cos(u) (4 + 3.8 cos(v))
y = sin(u) (4 + 3.8 cos(v))
z = (cos(v) + sin(v) - 1) (1 + sin(v)) log(1 - pi v / 10) + 7.5 sin(v)
0 <= u <= 2 pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Apple(float u, float v)
{

       float x = cos(u) * (4 + 3.8 * cos(v));
       float y = sin(u) * (4 + 3.8 * cos(v));
       float z = (cos(v) + sin(v) - 1) * (1 + sin(v)) * log(1 - PI*v / 10) + 7.5 * sin(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Springs/////
x = (1 - r1 * cos(v)) * cos(u)
y = (1 - r1 * cos(v)) * sin(u)
z = r2 * [sin(v) + periodlength * u / pi]
*///////////////////////////////////////////////////////////////////////////////

float3 Springs(float u, float v)
{

       float x = (1-param1 * cos(v)) * cos(u);
       float y = (1-param1 * cos(v)) * sin(u);
       float z = param2 * (sin(v) + param3 * u /PI);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Tear Drop/////
x = a(1- cos(u)) sin(u) cos(v)
y = a (1- cos(u)) sin(u) sin(v)
z =  cos(u)
0 <= u <= 2pi, 0 <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Tear(float u, float v)
{

       float x = param1 * (1- cos(u)) * sin(u) * cos(v);
       float y = param1 * (1- cos(u)) * sin(u) * sin(v);
       float z = cos(u);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Moebius strip/////
x = cos(s)+t*cos(s/2) * cos(s)
y = sin(s)+t*cos(s/2) * sin(s)
z = t*sin(s/2)
*///////////////////////////////////////////////////////////////////////////////

float3 Moebius(float u, float v)
{

       float x = cos(u) + v * cos(u/2) * cos(u);
       float y = sin(u) + v * cos(u/2) * sin(u);
       float z = v * sin(u/2);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Horn/////
x = (2 + u cos(v)) sin(2 pi u)
y = (2 + u cos(v)) cos(2 pi u) + 2 u
z = u sin(v)
0 <= u <= 1, 0 <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Horn(float u, float v)
{

       float x = (2 + u * cos(v)) * sin(TWOPI * u);
       float y = (2 + u * cos(v)) * cos(TWOPI * u) + 2 * u ;
       float z = u * sin(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Cresent/////
x = (2 + sin(2 pi u) sin(2 pi v)) sin(3 pi v)
y = (2 + sin(2 pi u) sin(2 pi v)) cos(3 pi v)
z = cos(2 pi u) sin(2 pi v) + 4 v - 2
0 <= u <= 1, 0 <= v <= 1
*///////////////////////////////////////////////////////////////////////////////

float3 Cresent(float u, float v)
{

       float x = (2 + sin(TWOPI * u) * sin(TWOPI * v)) * sin(3 * PI * v);
       float y = (2 + sin(TWOPI * u) * sin(TWOPI * v)) * cos(3 * PI * v);
       float z = cos(TWOPI * u) * sin(TWOPI * v) + 4 * v - 2;

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Slippers surface/////
x = (2 + cos(u)) cos(v)3 sin(v)
y = (2 + cos(u+2pi/3)) cos(2pi/3+v)2 sin(2pi/3+v)2
z = -(2 + cos(u-2pi/3)) cos(2pi/3-v)2 sin(2pi/3-v)3
0 <= u <= 2 pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 Slippers(float u, float v)
{

       float x = (2 + cos(u)) * pow(cos(v),3) * sin(v);
       float y = (2 + cos(u+TWOPI/3)) * pow(cos(TWOPI/3+v),2) * pow(sin(TWOPI/3+v),2);
       float z = -(2 + cos(u-TWOPI/3)) * pow(cos(TWOPI/3-v),2) * pow(sin(TWOPI/3-v),3);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Kuen Surface/////
x= (2 (cos(s)+ S sin(s)) sin(t)) / (1+s2 sin (t)2)
y= (2(sin(s)- s cos(s)) sin(t)) / (1+s2 sin (t)2)
z+ log(tan(t/2)) + ((2 cos(t))/(1+s2 sin (t)2)
-4,5<s<4,5, 0<t<PI
*///////////////////////////////////////////////////////////////////////////////

float3 Kuen(float u, float v)
{

       float x = (2 * (cos(v) + (v * sin(v))) * sin(u)) / (1 + pow(v,2) * pow(sin(u),2));
       float y = (2 * (sin(v) - (v * cos(v))) * sin(u)) / (1 + pow(v,2) * pow(sin(u),2));
       float z = (log(tan(u/2)) + ((2*cos(u)))) / (1 + pow(v,2) * pow(sin(u),2));

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Mathematical Shell/////
x= a(1- t/2PI)cos (nt)(1+cos(s))+c cos(nt)
y= a(1- t/2PI)sin (nt)(1+cos(s))+c sin(nt)
z= b(t/2PI) + a(1-t/2PI)sin(s)
0<s<2PI, 0<t<2PI
*///////////////////////////////////////////////////////////////////////////////

float3 Shell(float u, float v)
{

       float x = param1 * (1 - (u / (TWOPI))) * cos(param4*u) * (1 + cos(v)) + param3 * cos(param4*u);
       float y = param1 * (1 - (u / (TWOPI))) * sin(param4*u) * (1 + cos(v)) + param3 * sin(param4*u);
       float z = param2 * (u / (TWOPI)) + param1 * (1 - (u / (TWOPI))) * sin(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Pisot Triaxial/////
x = 0.655866 cos(1.03002 + u) (2 + cos(v))
y = 0.754878 cos(1.40772 - u) (2 + 0.868837 cos(2.43773 + v))
z = 0.868837 cos(2.43773 + u) (2 + 0.495098 cos(0.377696 - v))
0 <= u <= 2 pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 PisotTriaxial(float u, float v)
{
      float x = 0.655866 * cos(1.03002 + u) * (2 + cos(v));
      float y = 0.754878 * cos(1.40772 - u) * (2 + 0.868837 * cos(2.43773 + v));
      float z = 0.868837 * cos(2.43773 + u) * (2 + 0.495098 * cos(0.377696 - v));

      return float3(x, y, z);
}


////////////////////////////////////////////////////////////////////////////////
/*Klein Cycloid/////
x = cos(u/c) * cos(u/b) * (a + cos(v)) + sin(u/b) * sin(v) * cos(v)
y = sin(u/c) * cos(u/b) * (a + cos(v)) + sin(u/b) * sin(v) * cos(v)
z = -sin(u/b) * (a + cos(v)) + cos(u/b) * sin(v) * cos(v)
0 <= u <= 2 b c PI, 0 <= v <= 4 PI
*///////////////////////////////////////////////////////////////////////////////

float3 KleinCycloid(float u, float v)
{

      float x = cos(u/param3) * cos(u/param2) * (param1 + cos(v)) + sin(u/param2) * sin(v) * cos(v);
      float y = sin(u/param3) * cos(u/param2) * (param1 + cos(v)) + sin(u/param2) * sin(v) * cos(v);
      float z = -sin(u/param2) * (param1 + cos(v)) + cos(u/param2) * sin(v) * cos(v);

      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Klein Bottle/////
r = 4(1 - cos(u) / 2)
x = 6 cos(u)(1 + sin(u)) + r cos(v + PI)
y = 16 sin(u)
z = r sin(v)
pi <= u <= 2pi, pi <= v <= 2pi
*///////////////////////////////////////////////////////////////////////////////

float3 KleinBottle(float u, float v)
{

      float r = 4 * (1 - cos(u) / 2);
      float x = 6 * cos(u) * (1 + sin(u)) + r * cos(v + PI);
      float y = 16 * sin(u);
      float z = r * sin(v);
      
      return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Bour Minimal Surface/////
x = rn-1 cos((n-1) t)/(2 (n-1)) - rn+1 cos((n+1) t)/(2 (n+1))
y = rn-1 sin((n-1) t)/(2 (n-1)) + rn+1 sin((n+1) t)/(2 (n+1))
z = rn cos(n t) / n
0 <= r, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 Bour(float u, float v)
{

    float x = pow(u,param1-1)* cos((param1-1)*v)/(2 *(param1-1)) - pow(u,param1+1)* cos((param1+1) *v)/(2 *(param1+1));
    float y = pow(u,param1-1)* sin((param1-1)*v)/(2 *(param1-1)) + pow(u,param1+1)* sin((param1+1)* v)/(2* (param1+1));
    float z = pow(u,param1) * cos(param1 * v) / param1;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Figure8Torus/////
x = cos(u) (c + sin(v) cos(u) - sin(2 v) sin(u) / 2)
y = sin(u) (c + sin(v) cos(u) - sin(2 v) sin(u) / 2)
z = sin(u) sin(v) + cos(u) sin(2 v) / 2
-pi <= u,v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Figure8Torus(float u, float v)
{

    float x = cos(u) * (param3 + sin(v) * cos(u) - sin(2*v) * sin(u) / 2);
    float y = sin(u) * (param3 + sin(v) * cos(u) - sin(2*v) * sin(u) / 2) ;
    float z = sin(u) * sin(v) + cos(u) * sin(2*v) / 2;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Elliptic Torus/////
x = (c + cos(v)) cos(u)
y = (c + cos(v)) sin(u)
z = sin(v) + cos(v)
-pi <= u,v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 EllipticTorus(float u, float v)
{

    float x = (param3 + cos(v)) * cos(u);
    float y = (param3 + cos(v)) * sin(u) ;
    float z = sin(v) + cos(v);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Tractrix Based Minimal Surface/////
x = cos(u) (v - tanh(v))
y = cos(u) / cosh(v)
z = x2 - y2 + 2 x y tanh2(u)
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Tractrix(float u, float v)
{

    float x = cos(u) * (v - tanh(v));
    float y = cos(u) / cosh(v) ;
    float z = pow(x,2) - pow(y,2) + 2*x*y*pow(tanh(u),2);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Catenoid Minimal Surface/////
x = c cosh(v/c) cos(u)
y = c cosh(v/c) sin(u)
z = v
0 <= u <= 2pi, -infinity <= v <= infinity
*///////////////////////////////////////////////////////////////////////////////

float3 Catenoid(float u, float v)
{

    float x = param3 * cosh(v/param3) * cos(u);
    float y = param3 * cosh(v/param3) * sin(u) ;
    float z = v;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Helicoid Minimal Surface/////
x = c v cos(u)
y = c v sin(u)
z = u
-infinity <= u <= infinity, -infinity <= v <= infinity
*///////////////////////////////////////////////////////////////////////////////

float3 Helicoid(float u, float v)
{

    float x = param3 * v * cos(u);
    float y = param3 * v * sin(u) ;
    float z = u;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Scherk Minimal Surface/////
x = u
y = v
z = log(cos(c u) / cos(c v)) / c
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Scherk(float u, float v)
{

    float x = u;
    float y = v ;
    float z = log(cos(param3*u) / cos(param3*v)) / param3 ;

    return float3(x, y, z);
}


////////////////////////////////////////////////////////////////////////////////
/*Dini's Surface or Twisted Pseudosphere/////
x = a cos(u) sin(v)
y = a sin(u) sin(v)
z = a (cos(v) + log(tan(v/2))) + b u
0 <= u, 0 < v
*///////////////////////////////////////////////////////////////////////////////

float3 Dini(float u, float v)
{

    float x = param1 * cos(u) * sin(v);
    float y = param1 * sin(u) * sin(v) ;
    float z = param1 * (cos(v) + log(tan(v/2))) + param2 * u ;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Catalan Minimal Surface/////
x = c (u - cosh(v) sin(u))
y = c (1 - cosh(v) cos(u>)
z = -4 c sin(u/2) sinh(v/2)
-infinity <= u <= infinity,-infinity <= v <= infinity
*///////////////////////////////////////////////////////////////////////////////

float3 Catalan(float u, float v)
{

    float x = param3 * (u - cosh(v) * sin(u));
    float y = param3 * (1 - cosh(v) * cos(u)) ;
    float z = -4 * param3 * sin(u/2) * sinh(v/2) ;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Fish Surface/////
x = [cos(u) - cos(2 u)] cos(v) / 4
y = [sin(u) - sin(2 u)] sin(v) / 4
z = cos(u)
0 <= u <= pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 Fish(float u, float v)
{

    float x = (cos(u) - cos(2*u)) * cos(v) / 4;
    float y = (sin(u) - sin(2*u)) * sin(v) / 4 ;
    float z = cos(u) ;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Double Cone/////
x = v cos(u)
y = (v - 1) cos(u + 2 pi / 3)
z = (1 - v) cos(u - 2 pi / 3)
0 < u < 2pi, -1 < v < 1
*///////////////////////////////////////////////////////////////////////////////

float3 DoubleCone(float u, float v)
{

    float x = v * cos(u) ;
    float y = (v - 1) * cos(u + TWOPI/3) ;
    float z = (1 - v ) * cos(u - TWOPI/3) ;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Triaxial Teardrop/////
x = [1 - cos(u)] cos(u + 2 pi / 3) cos(v + 2 pi / 3) / 2;
y = [1 - cos(u)] cos(u + 2 pi /3) cos(v - 2 pi / 3) / 2;
z = cos(u - 2 pi / 3);
0 <= u <= pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 TriaxialTeardrop(float u, float v)
{

    float x = (1 - cos(u)) * cos(u + TWOPI / 3) * cos(v + TWOPI / 3) / 2;
    float y = (1 - cosh(u)) * cos(u + TWOPI /3) * cos(v - TWOPI / 3) / 2;
    float z = cos(u - TWOPI / 3) ;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Jet Surface/////
x = [1 - cosh(u)] sin(u) cos(v) / 2
y = [1 - cosh(u)] sin(u) sin(v) / 2
z = cosh(u)
0 <= u <= pi, 0 <= v <= 2 pi
*///////////////////////////////////////////////////////////////////////////////

float3 Jet(float u, float v)
{

    float x = (1 - cosh(u)) * sin(u) * cos(v)/2;
    float y = (1 - cosh(u)) * sin(u) * sin(v)/2;
    float z = cosh(u) ;

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Bent Horns/////
x = (2 + cos(u)) (v/3 - sin(v))
y = (2 + cos(u - 2 PI / 3)) (cos(v) - 1)
z = (2 + cos(u + 2 PI / 3)) (cos(v) - 1)
-pi <= u <= pi,-2pi <= v <= 2pi
*///////////////////////////////////////////////////////////////////////////////

float3 BentHorns(float u, float v)
{

    float x = (2 + cos(u)) * (v/3 - sin(v));
    float y = (2 + cos(u - TWOPI/3)) * (cos(v) - 1);
    float z = (2 + cos(u + TWOPI/3)) * (cos(v) - 1);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Ennepers Surface/////
x = u - u3 / 3 + u v2
y = v - v3 / 3 + v u2
z = u2 - v2
-2 <= u <= 2, -2 <= v <= 2
*///////////////////////////////////////////////////////////////////////////////

float3 Ennepers(float u, float v)
{

    float x = u - pow(u,3)/3 + u * pow(v,2);
    float y = v - pow(v,3)/3 + v * pow(u,2);
    float z = pow(u,2) - pow(v,2);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Pillow shape/////
x = cos(u)
y = cos(v)
z = sin(u) sin(v)
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Pillow(float u, float v)
{

    float x = cos(u);
    float y = cos(v);
    float z = sin(u) * sin(v);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Monkey Saddle/////
x = u
y = v
z = u3 - 3 u v2
-1.1 <= u <= 1.1, -1.1 <= v <= 1.1
*///////////////////////////////////////////////////////////////////////////////

float3 MonkeySaddle(float u, float v)
{

    float x = u;
    float y = v;
    float z = pow(u,3) - 3 * u * pow(v,2);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Kidney Surface/////
x = cos(u) (3 cos(v) - cos(3 v))
y = sin(u) (3 cos(v) - cos(3 v))
z = 3 sin(v) - sin(3 v)
0 <= u <= 2 pi, -pi/2 <= v <= pi/2
*///////////////////////////////////////////////////////////////////////////////

float3 Kidney(float u, float v)
{

    float x = cos(u) * (3*cos(v) - cos(3*v));
    float y = sin(u) * (3*cos(v) - cos(3*v));
    float z = 3 * sin(v) - sin(3*v);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/* Henneburg Minimal Surface/////
x = 2 sinh(u) cos(v) - 2 sinh(3 u) cos(3 v) / 3
y = 2 sinh(u) sin(v) + 2 sinh(3 u) sin(3 v) / 3
z = 2 cosh(2 u) cos(2 v)
-infinity <= u <= infinity, -infinity <= v <= infinity
*///////////////////////////////////////////////////////////////////////////////

float3 Henneburg(float u, float v)
{

    float x = 2 * sinh(u) * cos(v) - 2 * sinh(3*u) * cos(3*v)/3;
    float y = 2 * sinh(u) * sin(v) + 2 * sinh(3*u) * sin(3*v)/3;
    float z = 2 * cosh(2*u) * cos(2*v);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/* Snail Surface/////
x = u cos(v) sin(u)
y = u cos(u) cos(v)
z = -u sin(v)
0 <= u <= 2 pi, -pi/2 <= v <= pi/2
*///////////////////////////////////////////////////////////////////////////////

float3 Snail(float u, float v)
{

    float x = u * cos(v) * sin(u);
    float y = u * cos(u) * cos(v);
    float z = -u * sin(v);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Lemniscape/////
x = cos(v) sqrt(|sin(2 u)|) cos(u)
y = cos(v) sqrt(|sin(2 u)|) sin(u)
z = x2 - y2 + 2 x y tan2(v)
0 <= u <= pi, 0 <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Lemniscape(float u, float v)
{

    float cosvSqrtAbsSin2u = cos(v)*sqrt(abs(sin(2*u)));
    float x = cosvSqrtAbsSin2u*cos(u);
    float y = cosvSqrtAbsSin2u*sin(u);
    float z = pow(x,2) - pow(y,2) + 2 * x * y * pow(tan(v),2);

    return float3(x, y, z);
}

////////////////////////////////////////////////////////////////////////////////
/*Tranguloid Trefoil/////
x = 2 sin(3 u) / (2 + cos(v))
y = 2 (sin(u) + 2 sin(2 u)) / (2 + cos(v + 2 pi / 3))
z = (cos(u) - 2 cos(2 u)) (2 + cos(v)) (2 + cos(v + 2 pi / 3)) / 4
-pi <= u <= pi, -pi <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Tranguloid(float u, float v)
{

   float3 p;

   p.x = sin(3*u) * 2 / (2 + cos(v));
   p.y = (sin(u) + 2 * sin(2*u)) * 2 / (2 + cos(v + TWOPI / param1));
   p.z = (cos(u) - 2 * cos(2*u)) * (2 + cos(v)) * ((2 + cos(v + TWOPI/3))*0.25);

   return p;
}

////////////////////////////////////////////////////////////////////////////////

float3 Cone(float u, float theta)
{
   float3 p;
   float a = ((gridScaleX-u) / gridScaleX) * param1;// * (param2 * (cos(u*param3)+1) + 1);

   float3 offset;

   p.x = a * cos(theta);
   p.y = a * sin(theta);
   p.z = u ;

   offset.x = u * cos(param2*TWOPI - u);
   offset.y = u * cos(param3*TWOPI - u);
   offset.z = 0;

   return p+offset*param4;
}

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS_SteinbachScrew(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 4;
    gridScaleY *= 6,25;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =SteinbachScrew(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Sine(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Sine(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Paraboloid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Paraboloid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_HyperHelicoid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =HyperHelicoid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Hyperboloid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Hyperboloid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Eight(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI/2;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Eight(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Corkscrew(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Corkscrew(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Roman2Boy(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Roman2Boy(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_TwistedPipe(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =TwistedPipe(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Stiletto(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Stiletto(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_BohemianDome(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =BohemianDome(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Folium(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Folium(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_MaedersOwl(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 2*TWOPI;
    gridScaleY *= 1;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =MaedersOwl(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_AstroidalEllipsoid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =AstroidalEllipsoid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_TriaxialTritorus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =TriaxialTritorus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_LimpetTorus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =LimpetTorus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_TriaxialHexatorus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =TriaxialHexatorus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_CrossCap(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI/2;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =CrossCap(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Handkerchief(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Handkerchief(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Bow(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Bow(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Apple(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Apple(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Springs(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= 0.898;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Springs(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Tear(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Tear(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Moebius(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= 0.4;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Moebius(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Horn(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1,5;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Horn(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Cresent(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Cresent(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Slippers(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Slippers(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Kuen(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Kuen(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Shell(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 2*TWOPI;
    gridScaleY *= TWOPI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Shell(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_PisotTriaxial(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =PisotTriaxial(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_KleinCycloid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =KleinCycloid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_KleinBottle(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= TWOPI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =KleinBottle(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Bour(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Bour(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Figure8Torus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= -PI;
    gridScaleY *= PI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Figure8Torus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_EllipticTorus(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =EllipticTorus(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Tractrix(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Tractrix(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Catenoid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= 1 ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Catenoid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Helicoid(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1 ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Helicoid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Dini(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Dini(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Scherk(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Scherk(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

vs2ps VS_Catalan(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1 ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Catalan(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

////////////////////////////////////////////////////////////////////////////////

vs2ps VS_Fish(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Fish(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_DoubleCone(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= 1 ;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =DoubleCone(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_TriaxialTeardrop(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =TriaxialTeardrop(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_Jet(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Jet(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_BentHorns(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= TWOPI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz = BentHorns(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_Ennepers(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 2;
    gridScaleY *= 2;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Ennepers(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_Pillow(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Pillow(u, v);
    PosO.w = 1;
    
    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_MonkeySaddle(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;
    
    //map u and v
    gridScaleX *= 1.1;
    gridScaleY *= 1.1;

    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz = MonkeySaddle(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////////////////////////////

vs2ps VS_Kidney(
    float4 PosO: POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI/2;
    
    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;

    Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Kidney(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

///////////////////////////////////////////////////

vs2ps VS_Henneburg(
    float4 PosO: POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;
    
    //map u and v
    gridScaleX *= 1;
    gridScaleY *= 1;
    
    u = (PosO.x + gridOffsetX) * gridScaleX;
    v = (PosO.y + gridOffsetY) * gridScaleY;
    
    Out.UV  = float2(u, v);
    
    //get position
    PosO.xyz = Henneburg(u, v);
    PosO.w = 1;
    
    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

///////////////////////////////////////////////////

vs2ps VS_Snail(
    float4 PosO: POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= TWOPI;
    gridScaleY *= PI/2;

    u = PosO.x * gridScaleX + gridOffsetX;
    v = PosO.y * gridScaleY + gridOffsetY;

    Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Snail(u, v);
    PosO.w = 1;
    
    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

///////////////////////////////////////////////////

vs2ps VS_Lemniscape(
    float4 PosO: POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = PosO.x * gridScaleX + gridOffsetX;
    v = PosO.y * gridScaleY + gridOffsetY;

    Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Lemniscape(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

///////////////////////////////////////////////////

vs2ps VS_Tranguloid(
    float4 PosO: POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= PI;
    gridScaleY *= PI;

    u = PosO.x * gridScaleX + gridOffsetX;
    v = PosO.y * gridScaleY + gridOffsetY;

    Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Tranguloid(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

//////////////////////////////////////////////

vs2ps VS_Cone(
    float4 PosO: POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);
    
    float x, y, z, u, v, pi;

    //map u and v
    gridScaleX *= 1;
    gridScaleY *= TWOPI;

    u = PosO.x * gridScaleX + gridOffsetX;
    v = PosO.y * gridScaleY + gridOffsetY;

    Out.UV  = float2(u, v);

    //get position
    PosO.xyz = Cone(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------
float4 PS_SteinbachScrew(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (SteinbachScrew(u1, In.UV.y) - SteinbachScrew(u2, In.UV.y));
    bitang = (SteinbachScrew(In.UV.x, v1) - SteinbachScrew(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Sine(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Sine(u1, In.UV.y) - Sine(u2, In.UV.y));
    bitang = (Sine(In.UV.x, v1) - Sine(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Paraboloid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Paraboloid(u1, In.UV.y) - Paraboloid(u2, In.UV.y));
    bitang = (Paraboloid(In.UV.x, v1) - Paraboloid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_HyperHelicoid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (HyperHelicoid(u1, In.UV.y) - HyperHelicoid(u2, In.UV.y));
    bitang = (HyperHelicoid(In.UV.x, v1) - HyperHelicoid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Hyperboloid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Hyperboloid(u1, In.UV.y) - Hyperboloid(u2, In.UV.y));
    bitang = (Hyperboloid(In.UV.x, v1) - Hyperboloid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Eight(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Eight(u1, In.UV.y) - Eight(u2, In.UV.y));
    bitang = (Eight(In.UV.x, v1) - Eight(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Corkscrew(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Corkscrew(u1, In.UV.y) - Corkscrew(u2, In.UV.y));
    bitang = (Corkscrew(In.UV.x, v1) - Corkscrew(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Roman2Boy(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Roman2Boy(u1, In.UV.y) - Roman2Boy(u2, In.UV.y));
    bitang = (Roman2Boy(In.UV.x, v1) - Roman2Boy(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_TwistedPipe(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (TwistedPipe(u1, In.UV.y) - TwistedPipe(u2, In.UV.y));
    bitang = (TwistedPipe(In.UV.x, v1) - TwistedPipe(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Stiletto(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Stiletto(u1, In.UV.y) - Stiletto(u2, In.UV.y));
    bitang = (Stiletto(In.UV.x, v1) - Stiletto(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_BohemianDome(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (BohemianDome(u1, In.UV.y) - BohemianDome(u2, In.UV.y));
    bitang = (BohemianDome(In.UV.x, v1) - BohemianDome(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Folium(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Folium(u1, In.UV.y) - Folium(u2, In.UV.y));
    bitang = (Folium(In.UV.x, v1) - Folium(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_MaedersOwl(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (MaedersOwl(u1, In.UV.y) - MaedersOwl(u2, In.UV.y));
    bitang = (MaedersOwl(In.UV.x, v1) - MaedersOwl(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_AstroidalEllipsoid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (AstroidalEllipsoid(u1, In.UV.y) - AstroidalEllipsoid(u2, In.UV.y));
    bitang = (AstroidalEllipsoid(In.UV.x, v1) - AstroidalEllipsoid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_TriaxialTritorus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (TriaxialTritorus(u1, In.UV.y) - TriaxialTritorus(u2, In.UV.y));
    bitang = (TriaxialTritorus(In.UV.x, v1) - TriaxialTritorus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_LimpetTorus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (LimpetTorus(u1, In.UV.y) - LimpetTorus(u2, In.UV.y));
    bitang = (LimpetTorus(In.UV.x, v1) - LimpetTorus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_TriaxialHexatorus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (TriaxialHexatorus(u1, In.UV.y) - TriaxialHexatorus(u2, In.UV.y));
    bitang = (TriaxialHexatorus(In.UV.x, v1) - TriaxialHexatorus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_CrossCap(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (CrossCap(u1, In.UV.y) - CrossCap(u2, In.UV.y));
    bitang = (CrossCap(In.UV.x, v1) - CrossCap(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Handkerchief(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Handkerchief(u1, In.UV.y) - Handkerchief(u2, In.UV.y));
    bitang = (Handkerchief(In.UV.x, v1) - Handkerchief(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Bow(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Bow(u1, In.UV.y) - Bow(u2, In.UV.y));
    bitang = (Bow(In.UV.x, v1) - Bow(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Apple(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Apple(u1, In.UV.y) - Apple(u2, In.UV.y));
    bitang = (Apple(In.UV.x, v1) - Apple(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Springs(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Springs(u1, In.UV.y) - Springs(u2, In.UV.y));
    bitang = (Springs(In.UV.x, v1) - Springs(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Tear(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Tear(u1, In.UV.y) - Tear(u2, In.UV.y));
    bitang = (Tear(In.UV.x, v1) - Tear(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Moebius(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Moebius(u1, In.UV.y) - Moebius(u2, In.UV.y));
    bitang = (Moebius(In.UV.x, v1) - Moebius(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Horn(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Horn(u1, In.UV.y) - Horn(u2, In.UV.y));
    bitang = (Horn(In.UV.x, v1) - Horn(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Cresent(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Cresent(u1, In.UV.y) - Cresent(u2, In.UV.y));
    bitang = (Cresent(In.UV.x, v1) - Cresent(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Slippers(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Slippers(u1, In.UV.y) - Slippers(u2, In.UV.y));
    bitang = (Slippers(In.UV.x, v1) - Slippers(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Kuen(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Kuen(u1, In.UV.y) - Kuen(u2, In.UV.y));
    bitang = (Kuen(In.UV.x, v1) - Kuen(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Shell(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Shell(u1, In.UV.y) - Shell(u2, In.UV.y));
    bitang = (Shell(In.UV.x, v1) - Shell(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_PisotTriaxial(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (PisotTriaxial(u1, In.UV.y) - PisotTriaxial(u2, In.UV.y));
    bitang = (PisotTriaxial(In.UV.x, v1) - PisotTriaxial(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_KleinCycloid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (KleinCycloid(u1, In.UV.y) - KleinCycloid(u2, In.UV.y));
    bitang = (KleinCycloid(In.UV.x, v1) - KleinCycloid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_KleinBottle(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (KleinBottle(u1, In.UV.y) - KleinBottle(u2, In.UV.y));
    bitang = (KleinBottle(In.UV.x, v1) - KleinBottle(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Bour(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Bour(u1, In.UV.y) - Bour(u2, In.UV.y));
    bitang = (Bour(In.UV.x, v1) - Bour(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Figure8Torus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Figure8Torus(u1, In.UV.y) - Figure8Torus(u2, In.UV.y));
    bitang = (Figure8Torus(In.UV.x, v1) - Figure8Torus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_EllipticTorus(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (EllipticTorus(u1, In.UV.y) - EllipticTorus(u2, In.UV.y));
    bitang = (EllipticTorus(In.UV.x, v1) - EllipticTorus(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Tractrix(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Tractrix(u1, In.UV.y) - Tractrix(u2, In.UV.y));
    bitang = (Tractrix(In.UV.x, v1) - Tractrix(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Catenoid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Catenoid(u1, In.UV.y) - Catenoid(u2, In.UV.y));
    bitang = (Catenoid(In.UV.x, v1) - Catenoid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Helicoid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Helicoid(u1, In.UV.y) - Helicoid(u2, In.UV.y));
    bitang = (Helicoid(In.UV.x, v1) - Helicoid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Scherk(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Scherk(u1, In.UV.y) - Scherk(u2, In.UV.y));
    bitang = (Scherk(In.UV.x, v1) - Scherk(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Dini(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Dini(u1, In.UV.y) - Dini(u2, In.UV.y));
    bitang = (Dini(In.UV.x, v1) - Dini(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Catalan(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Catalan(u1, In.UV.y) - Catalan(u2, In.UV.y));
    bitang = (Catalan(In.UV.x, v1) - Catalan(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

 float4 PS_Fish(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Fish(u1, In.UV.y) - Fish(u2, In.UV.y));
    bitang = (Fish(In.UV.x, v1) - Fish(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_DoubleCone(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (DoubleCone(u1, In.UV.y) - DoubleCone(u2, In.UV.y));
    bitang = (DoubleCone(In.UV.x, v1) - DoubleCone(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_TriaxialTeardrop(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (TriaxialTeardrop(u1, In.UV.y) - TriaxialTeardrop(u2, In.UV.y));
    bitang = (TriaxialTeardrop(In.UV.x, v1) - TriaxialTeardrop(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Jet(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Jet(u1, In.UV.y) - Jet(u2, In.UV.y));
    bitang = (Jet(In.UV.x, v1) - Jet(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_BentHorns(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (BentHorns(u1, In.UV.y) - BentHorns(u2, In.UV.y));
    bitang = (BentHorns(In.UV.x, v1) - BentHorns(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Ennepers(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Ennepers(u1, In.UV.y) - Ennepers(u2, In.UV.y));
    bitang = (Ennepers(In.UV.x, v1) - Ennepers(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Pillow(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Pillow(u1, In.UV.y) - Pillow(u2, In.UV.y));
    bitang = (Pillow(In.UV.x, v1) - Pillow(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_MonkeySaddle(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (MonkeySaddle(u1, In.UV.y) - MonkeySaddle(u2, In.UV.y));
    bitang = (MonkeySaddle(In.UV.x, v1) - MonkeySaddle(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Kidney(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Kidney(u1, In.UV.y) - Kidney(u2, In.UV.y));
    bitang = (Kidney(In.UV.x, v1) - Kidney(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Henneburg(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Henneburg(u1, In.UV.y) - Henneburg(u2, In.UV.y));
    bitang = (Henneburg(In.UV.x, v1) - Henneburg(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Snail(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Snail(u1, In.UV.y) - Snail(u2, In.UV.y));
    bitang = (Snail(In.UV.x, v1) - Snail(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Lemniscape(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Lemniscape(u1, In.UV.y) - Lemniscape(u2, In.UV.y));
    bitang = (Lemniscape(In.UV.x, v1) - Lemniscape(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Tranguloid(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Tranguloid(u1, In.UV.y) - Tranguloid(u2, In.UV.y));
    bitang = (Tranguloid(In.UV.x, v1) - Tranguloid(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

///////////////////////////////////////////////////////////////////////////////////

float4 PS_Cone(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX; //(In.PosO.x + gridSpaceX + gridOffsetX) * gridScaleX;
    u2 = In.UV.x - gridSpaceX; //(In.PosO.x - gridSpaceX + gridOffsetX) * gridScaleX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY; //(In.PosO.y + gridSpaceY + gridOffsetY) * gridScaleY;
    v2 = In.UV.y - gridSpaceY; //(In.PosO.y - gridSpaceY + gridOffsetY) * gridScaleY;

    //get tangents
    tang   = (Cone(u1, In.UV.y) - Cone(u2, In.UV.y));
    bitang = (Cone(In.UV.x, v1) - Cone(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tV);
    NormV = normalize(NormO);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------
technique SteinbachScrew
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_SteinbachScrew();
        PixelShader = compile ps_2_a PS_SteinbachScrew();
    }
}

technique Sine
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Sine();
        PixelShader = compile ps_2_a PS_Sine();
    }
}

technique Paraboloid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Paraboloid();
        PixelShader = compile ps_2_a PS_Paraboloid();
    }
}

technique HyperHelicoid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_HyperHelicoid();
        PixelShader = compile ps_2_a PS_HyperHelicoid();
    }
}

technique Hyperboloid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Hyperboloid();
        PixelShader = compile ps_2_a PS_Hyperboloid();
    }
}

technique Eight
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Eight();
        PixelShader = compile ps_2_a PS_Eight();
    }
}

technique Corkscrew
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Corkscrew();
        PixelShader = compile ps_2_a PS_Corkscrew();
    }
}

technique Roman2Boy
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Roman2Boy();
        PixelShader = compile ps_2_a PS_Roman2Boy();
    }
}

technique TwistedPipe
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_TwistedPipe();
        PixelShader = compile ps_2_a PS_TwistedPipe();
    }
}

technique Stiletto
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Stiletto();
        PixelShader = compile ps_2_a PS_Stiletto();
    }
}

technique BohemianDome
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_BohemianDome();
        PixelShader = compile ps_2_a PS_BohemianDome();
    }
}

technique Folium
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Folium();
        PixelShader = compile ps_2_a PS_Folium();
    }
}

technique MaedersOwl
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_MaedersOwl();
        PixelShader = compile ps_2_a PS_MaedersOwl();
    }
}

technique AstroidalEllipsoid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_AstroidalEllipsoid();
        PixelShader = compile ps_2_a PS_AstroidalEllipsoid();
    }
}

technique TriaxialTritorus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_TriaxialTritorus();
        PixelShader = compile ps_2_a PS_TriaxialTritorus();
    }
}

technique LimpetTorus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_LimpetTorus();
        PixelShader = compile ps_2_a PS_LimpetTorus();
    }
}

technique TriaxialHexatorus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_TriaxialHexatorus();
        PixelShader = compile ps_2_a PS_TriaxialHexatorus();
    }
}

technique CrossCap
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_CrossCap();
        PixelShader = compile ps_2_a PS_CrossCap();
    }
}

technique Handkerchief
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Handkerchief();
        PixelShader = compile ps_2_a PS_Handkerchief();
    }
}

technique Bow
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Bow();
        PixelShader = compile ps_2_a PS_Bow();
    }
}

technique Apple
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Apple();
        PixelShader = compile ps_2_a PS_Apple();
    }
}

technique Springs
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Springs();
        PixelShader = compile ps_2_a PS_Springs();
    }
}

technique Tear
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Tear();
        PixelShader = compile ps_2_a PS_Tear();
    }
}

technique Moebius
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Moebius();
        PixelShader = compile ps_2_a PS_Moebius();
    }
}

technique Horn
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Horn();
        PixelShader = compile ps_2_a PS_Horn();
    }
}

technique Cresent
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Cresent();
        PixelShader = compile ps_2_a PS_Cresent();
    }
}

technique Slippers
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Slippers();
        PixelShader = compile ps_2_a PS_Slippers();
    }
}

technique Kuen
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Kuen();
        PixelShader = compile ps_2_a PS_Kuen();
    }
}

technique Shell
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Shell();
        PixelShader = compile ps_2_a PS_Shell();
    }
}

technique PisotTriaxial
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_PisotTriaxial();
        PixelShader = compile ps_2_a PS_PisotTriaxial();
    }
}

technique KleinCycloid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_KleinCycloid();
        PixelShader = compile ps_2_a PS_KleinCycloid();
    }
}

technique KleinBottle
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_KleinBottle();
        PixelShader = compile ps_2_a PS_KleinBottle();
    }
}

technique Bour
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Bour();
        PixelShader = compile ps_2_a PS_Bour();
    }
}

technique Figure8Torus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Figure8Torus();
        PixelShader = compile ps_2_a PS_Figure8Torus();
    }
}

technique EllipticTorus
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_EllipticTorus();
        PixelShader = compile ps_2_a PS_EllipticTorus();
    }
}

technique Tractrix
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Tractrix();
        PixelShader = compile ps_2_a PS_Tractrix();
    }
}

technique Catenoid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Catenoid();
        PixelShader = compile ps_2_a PS_Catenoid();
    }
}

technique Helicoid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Helicoid();
        PixelShader = compile ps_2_a PS_Helicoid();
    }
}

technique Scherk
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Scherk();
        PixelShader = compile ps_2_a PS_Scherk();
    }
}

technique Dini
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Dini();
        PixelShader = compile ps_2_a PS_Dini();
    }
}

technique Catalan
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Catalan();
        PixelShader = compile ps_2_a PS_Catalan();
    }
}

technique Fish
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Fish();
        PixelShader = compile ps_2_a PS_Fish();
    }
}

technique DoubleCone
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_DoubleCone();
        PixelShader = compile ps_2_a PS_DoubleCone();
    }
}

technique TriaxialTeardrop
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_TriaxialTeardrop();
        PixelShader = compile ps_2_a PS_TriaxialTeardrop();
    }
}

technique Jet
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Jet();
        PixelShader = compile ps_2_a PS_Jet();
    }
}

technique BentHorns
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_BentHorns();
        PixelShader = compile ps_2_a PS_BentHorns();
    }
}

technique Ennepers
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Ennepers();
        PixelShader = compile ps_2_a PS_Ennepers();
    }
}

technique Pillow
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Pillow();
        PixelShader = compile ps_2_a PS_Pillow();
    }
}

technique MonkeySaddle
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_MonkeySaddle();
        PixelShader = compile ps_2_a PS_MonkeySaddle();
    }
}

technique Kidney
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Kidney();
        PixelShader = compile ps_2_a PS_Kidney();
    }
}

technique Henneburg
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Henneburg();
        PixelShader = compile ps_2_a PS_Henneburg();
    }
}
technique Snail
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Snail();
        PixelShader = compile ps_2_a PS_Snail();
    }
}

technique Lemniscape
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Lemniscape();
        PixelShader = compile ps_2_a PS_Lemniscape();
    }
}

technique Tranguloid
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Tranguloid();
        PixelShader = compile ps_2_a PS_Tranguloid();
    }
}

technique Cone
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Cone();
        PixelShader = compile ps_2_a PS_Cone();
    }
}
